@@ -2,7 +2,7 @@
  * This file is part of 'literanger'. literanger was adapted from the 'ranger'
  * package for R Statistical Software <https://www.r-project.org>. ranger was
  * authored by Marvin N Wright with the GNU General Public License version 3.
- * The adaptation was performed by Stephen Wade in 2023. literanger carries the
+ * The adaptation was performed by stephematician in 2023. literanger carries the
  * same license, terms, and permissions as ranger.
  *
  * literanger is free software: you can redistribute it and/or modify
@@ -16,13 +16,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with literanger. If not, see <http://www.gnu.org/licenses/>.
+ * along with literanger. If not, see <https://www.gnu.org/licenses/>.
  *
  * Written by:
  *
- *   Stephen Wade
- *   Cancer Council New South Wales
- *   Woolloomooloo NSW 2011
+ *   stephematician
+ *   stephematician@gmail.com
  *   Australia
  *-------------------------------------------------------------------------------
  */
@@ -42,6 +41,7 @@
 
 /* general literanger headers */
 #include "enum_types.h"
+#include "globals.h"
 #include "utility.h"
 /* literanger R package headers */
 #include "cpp11_utility.h"
@@ -66,7 +66,9 @@ cpp11::list cpp11_predict(
     using nodes_matrix_t = cpp11::writable::integers_matrix<cpp11::by_row>;
     cpp11::writable::list result;
 
-    const std::string tree_type = cpp11::as_cpp<std::string>(object["tree_type"]);
+    const std::string tree_type = (
+        cpp11::as_cpp<std::string>(object["tree_type"])
+    );
     cpp11::external_pointer<ForestBase> forest = object["cpp11_ptr"];
     const size_t n_tree = forest->get_tree_parameters().size();
 
